@@ -8,21 +8,17 @@ import re
 import stat
 import shutil
 import tempfile
-
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch # py2
+from unittest.mock import patch
+import pytest
+import sys
 
 from jupyter_core import paths
 from jupyter_core.paths import (
     jupyter_config_dir, jupyter_data_dir, jupyter_runtime_dir,
-    jupyter_path, ENV_JUPYTER_PATH,
+    jupyter_path, jupyter_config_path, ENV_JUPYTER_PATH,
     secure_write, is_hidden, is_file_hidden
 )
-from ipython_genutils.tempdir import TemporaryDirectory
-from ipython_genutils.py3compat import cast_unicode
-from ipython_genutils.testing.decorators import skip_if_not_win32, skip_win32
+
 from .mocking import darwin, windows, linux
 
 pjoin = os.path.join
@@ -59,6 +55,16 @@ def realpath(path):
 home_jupyter = realpath('~/.jupyter')
 
 
+def test_envset():
+    true_values = ['', 'True', 'on', 'yes', 'Y', '1', 'anything']
+    false_values = ['n', 'No', 'N', 'fAlSE', '0', '0.0', 'Off']
+    with patch.dict('os.environ', ((f"FOO_{v}", v) for v in true_values + false_values)):
+        for v in true_values:
+            assert paths.envset(f"FOO_{v}")
+        for v in false_values:
+            assert not paths.envset(f"FOO_{v}")
+        assert not paths.envset("THIS_VARIABLE_SHOULD_NOT_BE_SET")
+
 def test_config_dir_darwin():
     with darwin, no_config_env:
         config = jupyter_config_dir()
@@ -174,6 +180,11 @@ def test_jupyter_path():
     assert path[0] == jupyter_data_dir()
     assert path[-2:] == system_path
 
+def test_jupyter_path_prefer_env():
+    with patch.dict('os.environ', {'JUPYTER_PREFER_ENV_PATH': 'true'}):
+        path = jupyter_path()
+    assert path[0] == paths.ENV_JUPYTER_PATH[0]
+    assert path[1] == jupyter_data_dir()
 
 def test_jupyter_path_env():
     path_env = os.pathsep.join([
@@ -197,9 +208,29 @@ def test_jupyter_path_subdir():
     for p in path:
         assert p.endswith(pjoin('', 'sub1', 'sub2'))
 
+def test_jupyter_config_path():
+    path = jupyter_config_path()
+    assert path[0] == jupyter_config_dir()
+    assert path[1] == paths.ENV_CONFIG_PATH[0]
+
+def test_jupyter_config_path_prefer_env():
+    with patch.dict('os.environ', {'JUPYTER_PREFER_ENV_PATH': 'true'}):
+        path = jupyter_config_path()
+    assert path[0] == paths.ENV_CONFIG_PATH[0]
+    assert path[1] == jupyter_config_dir()
+
+def test_jupyter_config_path_env():
+    path_env = os.pathsep.join([
+        pjoin('foo', 'bar'),
+        pjoin('bar', 'baz', ''), # trailing /
+    ])
+
+    with patch.dict('os.environ', {'JUPYTER_CONFIG_PATH': path_env}):
+        path = jupyter_config_path()
+    assert path[:2] == [pjoin('foo', 'bar'), pjoin('bar', 'baz')]
 
 def test_is_hidden():
-    with TemporaryDirectory() as root:
+    with tempfile.TemporaryDirectory() as root:
         subdir1 = os.path.join(root, 'subdir')
         os.makedirs(subdir1)
         assert not is_hidden(subdir1, root)
@@ -225,12 +256,11 @@ def test_is_hidden():
         assert not is_file_hidden(subdir56, os.stat(subdir56))
 
 
-@skip_if_not_win32
+@pytest.mark.skipif(sys.platform != "win32", reason="only run on windows")
 def test_is_hidden_win32():
     import ctypes
-    with TemporaryDirectory() as root:
-        root = cast_unicode(root)
-        subdir1 = os.path.join(root, u'subdir')
+    with tempfile.TemporaryDirectory() as root:
+        subdir1 = os.path.join(root, 'subdir')
         os.makedirs(subdir1)
         assert not is_hidden(subdir1, root)
         r = ctypes.windll.kernel32.SetFileAttributesW(subdir1, 0x02)
@@ -239,7 +269,7 @@ def test_is_hidden_win32():
         assert is_file_hidden(subdir1)
 
 
-@skip_if_not_win32
+@pytest.mark.skipif(sys.platform != "win32", reason="only runs on windows")
 def test_secure_write_win32():
     def fetch_win32_permissions(filename):
         '''Extracts file permissions on windows using icacls'''
@@ -264,7 +294,7 @@ def test_secure_write_win32():
         permissions = fetch_win32_permissions(fname)
         print(permissions) # for easier debugging
         assert username in permissions
-        assert permissions[username] == set(['r', 'w'])
+        assert permissions[username] == set(['r', 'w', 'd'])
         assert 'administrators' in permissions
         assert permissions['administrators'] == set(['f'])
         assert 'everyone' not in permissions
@@ -282,7 +312,7 @@ def test_secure_write_win32():
         shutil.rmtree(directory)
 
 
-@skip_win32
+@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
 def test_secure_write_unix():
     directory = tempfile.mkdtemp()
     fname = os.path.join(directory, 'check_perms')
@@ -290,7 +320,7 @@ def test_secure_write_unix():
         with secure_write(fname) as f:
             f.write('test 1')
         mode = os.stat(fname).st_mode
-        assert '0600' == oct(stat.S_IMODE(mode)).replace('0o', '0')
+        assert 0o0600 == (stat.S_IMODE(mode) & 0o7677)  # tolerate owner-execute bit
         with open(fname, 'r') as f:
             assert f.read() == 'test 1'
 
@@ -299,7 +329,7 @@ def test_secure_write_unix():
         with secure_write(fname) as f:
             f.write('test 2')
         mode = os.stat(fname).st_mode
-        assert '0600' == oct(stat.S_IMODE(mode)).replace('0o', '0')
+        assert 0o0600 == (stat.S_IMODE(mode) & 0o7677)  # tolerate owner-execute bit
         with open(fname, 'r') as f:
             assert f.read() == 'test 2'
     finally:
